@@ -3,9 +3,9 @@
 ## author: Brice Ozenne
 ## created: feb 17 2017 (10:06) 
 ## Version: 
-## last-updated: Dec  5 2020 (10:50) 
+## last-updated: Mar  7 2022 (08:31) 
 ##           By: Thomas Alexander Gerds
-##     Update #: 992
+##     Update #: 1275
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -66,31 +66,42 @@
 #' ## display baseline hazard
 #' e.basehaz <- predictCox(m.cox)
 #' autoplot(e.basehaz, type = "cumhazard")
+#' \dontrun{
+#' autoplot(e.basehaz, type = "cumhazard", size.point = 0) ## without points
 #' autoplot(e.basehaz, type = "cumhazard", smooth = TRUE)
 #' autoplot(e.basehaz, type = "cumhazard", smooth = TRUE, first.derivative = TRUE)
-#'
-#' ## display baseline hazard with type of event 
+#' }
+#' 
+#' ## display baseline hazard with type of event
+#' \dontrun{
 #' e.basehaz <- predictCox(m.cox, keep.newdata = TRUE)
 #' autoplot(e.basehaz, type = "cumhazard")
+#' autoplot(e.basehaz, type = "cumhazard", shape.point = c(3,NA))
+#' }
 #'
 #' ## display predicted survival
+#' \dontrun{
 #' pred.cox <- predictCox(m.cox, newdata = d[1:2,],
 #'   times = seqTau, type = "survival", keep.newdata = TRUE)
 #' autoplot(pred.cox)
 #' autoplot(pred.cox, smooth = TRUE)
 #' autoplot(pred.cox, group.by = "covariates")
 #' autoplot(pred.cox, group.by = "covariates", reduce.data = TRUE)
-#' 
+#' autoplot(pred.cox, group.by = "X1", reduce.data = TRUE)
+#' }
 #' 
 #' ## predictions with confidence interval/bands
+#' \dontrun{
 #' pred.cox <- predictCox(m.cox, newdata = d[1:2,,drop=FALSE],
 #'   times = seqTau, type = "survival", band = TRUE, se = TRUE, keep.newdata = TRUE)
 #' res <- autoplot(pred.cox, ci = TRUE, band = TRUE, plot = FALSE)
 #' res$plot + facet_wrap(~row)
 #' res2 <- autoplot(pred.cox, ci = TRUE, band = TRUE, alpha = 0.1, plot = FALSE)
 #' res2$plot + facet_wrap(~row)
-#'
+#' }
+#' 
 #' #### Stratified Cox model ####
+#' \dontrun{
 #' m.cox.strata <- coxph(Surv(time,event)~ strata(X1) + strata(X2) + X3 + X4,
 #'                       data = d, x = TRUE, y = TRUE)
 #'
@@ -98,9 +109,9 @@
 #' pred.baseline <- predictCox(m.cox.strata, keep.newdata = TRUE, type = "survival")
 #' res <- autoplot(pred.baseline)
 #' res$plot + facet_wrap(~strata, labeller = label_both)
-#' 
+#'
 #' ## predictions
-#' pred.cox.strata <- predictCox(m.cox.strata, newdata = d[1:5,,drop=FALSE],
+#' pred.cox.strata <- predictCox(m.cox.strata, newdata = d[1:3,,drop=FALSE],
 #'                               time = seqTau, keep.newdata = TRUE, se = TRUE)
 #'
 #' res2 <- autoplot(pred.cox.strata, type = "survival", group.by = "strata", plot = FALSE)
@@ -108,6 +119,26 @@
 #' 
 #' ## smooth version
 #' autoplot(pred.cox.strata, type = "survival", group.by = "strata", smooth = TRUE, ci = FALSE)
+#' }
+#' 
+#' #### Cox model with splines ####
+#' \dontrun{
+#' require(splines)
+#' m.cox.spline <- coxph(Surv(time,event)~ X1 + X2 + ns(X6,4),
+#'                 data = d, x = TRUE, y = TRUE)
+#' grid <- data.frame(X1 = factor(0,0:1), X2 = factor(0,0:1),
+#'                    X6 = seq(min(d$X6),max(d$X6), length.out = 100))
+#' pred.spline <- predictCox(m.cox.spline, newdata = grid, keep.newdata = TRUE,
+#'                           se = TRUE, band = TRUE, centered = TRUE, type = "lp")
+#' autoplot(pred.spline, group.by = "X6")
+#' autoplot(pred.spline, group.by = "X6", alpha = 0.5)
+#' 
+#' grid2 <- data.frame(X1 = factor(1,0:1), X2 = factor(0,0:1),
+#'                     X6 = seq(min(d$X6),max(d$X6), length.out = 100))
+#' pred.spline <- predictCox(m.cox.spline, newdata = rbind(grid,grid2), keep.newdata = TRUE,
+#'                           se = TRUE, band = TRUE, centered = TRUE, type = "lp")
+#' autoplot(pred.spline, group.by = c("X6","X1"), alpha = 0.5, plot = FALSE)$plot + facet_wrap(~X1)
+#' }
 
 ## * autoplot.predictCox (code)
 #' @rdname autoplot.predictCox
@@ -118,7 +149,7 @@ autoplot.predictCox <- function(object,
                                 ci = object$se,
                                 band = object$band,
                                 plot = TRUE,
-                                smooth = FALSE,
+                                smooth = NULL,
                                 digits = 2,
                                 alpha = NA,
                                 group.by = "row",
@@ -128,9 +159,10 @@ autoplot.predictCox <- function(object,
                                  ...){
   
     ## initialize and check    
-    possibleType <- c("cumhazard","survival")
+    possibleType <- c("cumhazard","survival","lp")
     possibleType <- possibleType[possibleType %in% names(object)]
-
+    nVar.lp <- length(object$var.lp)
+        
     if(is.null(type)){
         if(length(possibleType) == 1){
             type <- possibleType
@@ -140,30 +172,59 @@ autoplot.predictCox <- function(object,
     }else{
         type <- match.arg(type, possibleType)  
     }
+    if(is.null(smooth)){
+        if(type == "lp"){
+            smooth <- 0.5
+        }else{
+            smooth <- FALSE
+        }
+    }
     if(is.null(ylab)){
         if(first.derivative){
             ylab <- switch(type,
-                           "cumhazard" = if(object$baseline && object$nVar>0){"instantaneous baseline hazard"}else{"instantaneous hazard"},
-                           "survival" = if(object$baseline && object$nVar>0){"derivative of the baseline survival"}else{"derivative of the survival"})
+                           "cumhazard" = if(object$baseline && nVar.lp>0){"instantaneous baseline hazard"}else{"instantaneous hazard"},
+                           "survival" = if(object$baseline && nVar.lp>0){"derivative of the baseline survival"}else{"derivative of the survival"})
         }else{
             ylab <- switch(type,
-                           "cumhazard" = if(object$baseline && object$nVar>0){"cumulative baseline hazard"}else{"cumulative hazard"},
-                           "survival" = if(object$baseline && object$nVar>0){"baseline survival"}else{"survival"})
+                           "lp" = "linear predictor",
+                           "cumhazard" = if(object$baseline && nVar.lp>0){"cumulative baseline hazard"}else{"cumulative hazard"},
+                           "survival" = if(object$baseline && nVar.lp>0){"baseline survival"}else{"survival"})
         }
     }
 
-    group.by <- match.arg(group.by, c("row","covariates","strata"))
+    if(type=="lp"){
+        group.by <- match.arg(group.by, object$var.lp, several.ok = TRUE)
+    }else{
+        if(length(group.by)>1){
+            stop("Argument \'group.by\' must have length 1.\n")
+        }
+        group.by <- match.arg(group.by, c("row","covariates","strata",object$var.lp,object$var.strata))
+    }
+    
 
-    if((group.by[[1]] == "covariates") && ("newdata" %in% names(object)) == FALSE){
+    if(group.by[[1]] == "covariates" && ("newdata" %in% names(object)) == FALSE){
         stop("argument \'group.by\' cannot be \"covariates\" when newdata is missing in the object \n",
              "set argment \'keep.newdata\' to TRUE when calling the predictCox function \n")
     }
 
-    if(group.by[[1]] == "strata" && ("strata" %in% names(object) == FALSE)){
+    if(group.by[[1]] %in% c("strata") && ("strata" %in% names(object) == FALSE)){
         stop("argument \'group.by\' cannot be \"strata\" when strata is missing in the object \n",
              "set argment \'keep.strata\' to TRUE when calling the predictCox function \n")
     }
-  
+    if(group.by[[1]] %in% c(object$var.lp,object$var.strata) && ("newdata" %in% names(object) == FALSE)){
+        stop("argument \'group.by\' cannot be \"",group.by[[1]],"\" when newdata is missing in the object \n",
+             "set argment \'keep.newdata\' to TRUE when calling the predictCox function \n")
+    }
+    if(!is.null(object$newdata$strata) && !is.null(object$var.strata)){
+        if(any(object$var.strata %in% colnames(object$newdata))){
+            stop("cannot handle covariate \"",paste(object$var.strata[object$var.strata %in% colnames(object$newdata)],collapse ="\" \""),"\" as this name is used internally.\n",
+                 "consider refit the model using another name for the covariate. \n")
+        }
+        splitStrata <- do.call(rbind,strsplit(split = ",", x = as.character(object$newdata$strata), fixed = TRUE))
+        colnames(splitStrata) <- object$infoVar$stratavars.original
+        object$newdata <- cbind(object$newdata,splitStrata)
+    }
+        
     if(ci[[1]]==TRUE && (object$se[[1]]==FALSE || is.null(object$conf.level))){
         stop("argument \'ci\' cannot be TRUE when no standard error have been computed \n",
              "set arguments \'se\' and \'confint\' to TRUE when calling the predictCox function \n")
@@ -182,99 +243,144 @@ autoplot.predictCox <- function(object,
     ##     stop("unknown argument",txt.s,": \"",paste0(txt,collapse="\" \""),"\" \n")
     ## }
 
-    ## reshape data
-    if(!is.matrix(object[[type]])){
-        
-        ## baseline hazard/survival
-        if(is.null(object[["strata"]])){
-            object[[type]] <- rbind(object[[type]])
-            if(object$nTimes==0){
-                if(0 %in% object$times == FALSE){
-                    if(type=="cumhazard"){
-                        object[[type]] <- cbind(0,object[[type]])
-                    }else if(type=="survival"){
-                        object[[type]] <- cbind(1,object[[type]])
-                    }
-                    object$times <- c(0,object$times)
-                    if(!is.null(object$newdata)){
-                        object$newdata <- rbind(data.table(start = 0, stop = 0, status = NA, strata = 1, strata.num = 0, eXb = NA, statusM1 = NA, XXXindexXXX = NA),
-                                                object$newdata)
-                    }
-                }
-                if(object$lastEventTime %in% object$times == FALSE){
-                    object[[type]] <- cbind(object[[type]],utils::tail(object[[type]],1))
-                    object$times <- c(object$times,object$lastEventTime)
-                    if(!is.null(object$newdata)){
-                        object$newdata <- rbind(data.table(start = 0, stop = object$lastEventTime, status = 0, strata = 1, strata.num = 0, eXb = NA, statusM1 = NA, XXXindexXXX = NA),
-                                                object$newdata)
-                    }
-                }
-            }
+    ## ** reshape data
+    if(type == "lp"){
+        if(first.derivative){
+            stop("Argument \'first.derivative\' should be FALSE when argument \'type\' equals \"lp\". \n")
+        }
+        if(length(object$var.lp)==0){
+            stop("No covariate in the linear predictor so nothing to display.\n")
+        }
+        if(group.by[1] %in% object$var.lp == FALSE){
+            stop("The first element of argument \'group.by\' should refer to one of the covariates: \"",paste(object$var.lp, collapse= "\" \""),"\".\n")
+        }
+        if("time" %in% group.by){
+            stop("The argument \'group.by\' should not contain \"time\" as this name is used internally.\n")
+        }
+        if("row" %in% group.by){
+            stop("The argument \'group.by\' should not contain \"row\" as this name is used internally.\n")
+        }
+        if("lowerCI" %in% group.by){
+            stop("The argument \'group.by\' should not contain \"lowerCI\" as this name is used internally.\n")
+        }
+        if("upperCI" %in% group.by){
+            stop("The argument \'group.by\' should not contain \"upperCI\" as this name is used internally.\n")
+        }
+        dataL <- data.table::as.data.table(object)
+        dataL$lp.smooth <- dataL$lp
+        dataL[["time"]] <- dataL[[group.by[1]]]
+        dataL[["row"]] <- 1
+        if(ci){
+            data.table::setnames(dataL, old = c("lp.lower","lp.upper"), new = c("lowerCI","upperCI"))
+            dataL$lowerCI.smooth <- dataL$lowerCI
+            dataL$upperCI.smooth <- dataL$upperCI
+        }
+        if(band){
+            data.table::setnames(dataL, old = c("lp.lowerBand","lp.upperBand"), new = c("lowerBand","upperBand"))
+            dataL$lowerBand.smooth <- dataL$lowerBand
+            dataL$upperBand.smooth <- dataL$upperBand
+        }
+        object$infoVar$time <- group.by[1]
+        group.by <- if(length(group.by[-1])==0){"row"}else{group.by[-1]}
+    }else{
 
-        }else{
-            strata <- unique(object[["strata"]])
-            n.strata <- length(strata)
-            time <- unique(sort(object[["times"]])) 
-            n.time <- length(time)
-            type.tempo <- matrix(NA, nrow = n.strata, ncol = n.time)
+        if(!is.matrix(object[[type]])){
+        
+            ## baseline hazard/survival
+            if(is.null(object[["strata"]])){
+                object[[type]] <- rbind(object[[type]])
+                if(object$nTimes==0){
+                    if(0 %in% object$times == FALSE){
+                        if(type=="cumhazard"){
+                            object[[type]] <- cbind(0,object[[type]])
+                        }else if(type=="survival"){
+                            object[[type]] <- cbind(1,object[[type]])
+                        }                   
+                        object$times <- c(0,object$times)
+                        if(!is.null(object$newdata)){
+                            object$newdata <- rbind(data.table(start = 0, stop = 0, status = NA, strata = 1, strata.num = 0, eXb = NA, statusM1 = NA, XXXindexXXX = NA),
+                                                    object$newdata)
+                        }
+                    }
+                    if(object$lastEventTime %in% object$times == FALSE){
+                        object[[type]] <- cbind(object[[type]],object[[type]][length(object[[type]])])
+                        object$times <- c(object$times,pmin(object$lastEventTime,max(object$times)+1e-12))
+                        if(!is.null(object$newdata)){
+                            object$newdata <- rbind(data.table(start = 0, stop = object$lastEventTime, status = 0, strata = 1, strata.num = 0, eXb = NA, statusM1 = NA, XXXindexXXX = NA),
+                                                    object$newdata)
+                        }
+                    }
+                }
+
+            }else{
+                index.unique <- !duplicated(object$strata)
+                strata <- object$strata[index.unique]
+                if(!is.null(attr(object$strata,"covariates"))){
+                    attr(strata,"covariates") <- attr(object$strata,"covariates")[index.unique]
+                }
+                n.strata <- length(strata)
+                time <- unique(sort(object[["times"]])) 
+                n.time <- length(time)
+                type.tempo <- matrix(NA, nrow = n.strata, ncol = n.time)
 
             init <- switch(type,
                            "cumhazard" = 0,
                            "survival" = 1)
 
-            for(iStrata in 1:n.strata){ ## iStrata <- 1
-                index.strata <- which(object[["strata"]]==strata[iStrata])
-                type.tempo[iStrata,]  <- stats::approx(x = object[["times"]][index.strata],
-                                                       y = object[[type]][index.strata],
-                                                       yleft = init,
-                                                       yright = NA,
-                                                       xout = time,
-                                                       method = "constant")$y
+                for(iStrata in 1:n.strata){ ## iStrata <- 1
+                    index.strata <- which(object[["strata"]]==strata[iStrata])
+                    type.tempo[iStrata,]  <- stats::approx(x = object[["times"]][index.strata],
+                                                           y = object[[type]][index.strata],
+                                                           yleft = init,
+                                                           yright = NA,
+                                                           xout = time,
+                                                           method = "constant")$y
                 
             }
             object[[type]] <- type.tempo
             object[["strata"]] <- strata
             object[["times"]] <- time
-            group.by <- "strata"
         }
 
-        newdata <- NULL
-        if(object$nTimes==0){
-            status <- object$newdata
+            newdata <- NULL
+            if(object$nTimes==0){
+                status <- object$newdata
+            }else{
+                status <- NULL
+            }
         }else{
+            newdata <- data.table::copy(object$newdata) ## can be NULL
+            if(!is.null(newdata) && reduce.data[[1]]==TRUE){
+                test <- unlist(newdata[,lapply(.SD, function(col){length(unique(col))==1})])
+                if(any(test)){
+                    newdata[, (names(test)[test]):=NULL]
+                }        
+            }
             status <- NULL
-        }
-    }else{
-        newdata <- data.table::copy(object$newdata) ## can be NULL
-        if(!is.null(newdata) && reduce.data[[1]]==TRUE){
-            test <- unlist(newdata[,lapply(.SD, function(col){length(unique(col))==1})])
-            if(any(test)){
-                newdata[, (names(test)[test]):=NULL]
-            }        
-        }
-        status <- NULL
-        if(first.derivative && ci){
-            if(is.null(object$vcov[[type]])){
-                stop("Set argument \'iid\' to TRUE when calling predictCox to be able to display confidence intervals for the first derivative of the ",type,".\n")
+            if(first.derivative && ci){
+                if(is.null(object$vcov[[type]])){
+                    stop("Set argument \'iid\' to TRUE when calling predictCox to be able to display confidence intervals for the first derivative of the ",type,".\n")
+                }
+                attr(first.derivative,"vcov") <- object$vcov[[type]]
             }
-            attr(first.derivative,"vcov") <- object$vcov[[type]]
         }
-    }
 
-    dataL <- predict2melt(outcome = object[[type]], ci = ci, band = band,
-                          outcome.lower = if(ci){object[[paste0(type,".lower")]]}else{NULL},
-                          outcome.upper = if(ci){object[[paste0(type,".upper")]]}else{NULL},
-                          outcome.lowerBand = if(band){object[[paste0(type,".lowerBand")]]}else{NULL},
-                          outcome.upperBand = if(band){object[[paste0(type,".upperBand")]]}else{NULL},
-                          newdata = newdata,
-                          status = status,
-                          strata = object$strata,
-                          times = object$times,
-                          name.outcome = type,
-                          group.by = group.by,
-                          digits = digits
-                          )
-    ## display
+        dataL <- predict2melt(outcome = object[[type]], ci = ci, band = band,
+                              outcome.lower = if(ci){object[[paste0(type,".lower")]]}else{NULL},
+                              outcome.upper = if(ci){object[[paste0(type,".upper")]]}else{NULL},
+                              outcome.lowerBand = if(band){object[[paste0(type,".lowerBand")]]}else{NULL},
+                              outcome.upperBand = if(band){object[[paste0(type,".upperBand")]]}else{NULL},
+                              newdata = newdata,
+                              status = status,
+                              strata = object$strata,
+                              times = object$times,
+                              name.outcome = type,
+                              group.by = group.by,
+                              digits = digits
+                              )
+    }
+    
+    ## ** display
     gg.res <- predict2plot(dataL = dataL,
                            name.outcome = type,
                            ci = ci,
@@ -289,9 +395,9 @@ autoplot.predictCox <- function(object,
                            ...
                            )
   
-  if(plot){
-    print(gg.res$plot)
-  }
+    if(plot){
+        print(gg.res$plot)
+    }
   
     return(invisible(gg.res))
 }
@@ -309,9 +415,8 @@ predict2melt <- function(outcome, name.outcome,
         time.names <- times 
     }else{
         time.names <- 1:n.time
-    }    
+    }
     colnames(outcome) <- paste0(name.outcome,"_",time.names)
-    keep.cols <- unique(c("time",name.outcome,"row",group.by))
 
     ## merge outcome with CI and band ####
     pattern <- paste0(name.outcome,"_")
@@ -331,7 +436,6 @@ predict2melt <- function(outcome, name.outcome,
         outcome <- cbind(outcome, M.status)
         pattern <- c(pattern,"status")
     }
-
     if(ci){
         pattern <- c(pattern,"lowerCI_","upperCI_")
     
@@ -340,7 +444,6 @@ predict2melt <- function(outcome, name.outcome,
     }
     if(band){
         pattern <- c(pattern,"lowerBand_","upperBand_")
-        keep.cols <- c(keep.cols,"lowerBand","upperBand")
         
         colnames(outcome.lowerBand) <- paste0("lowerBand_",time.names)
         colnames(outcome.upperBand) <- paste0("upperBand_",time.names)
@@ -352,22 +455,38 @@ predict2melt <- function(outcome, name.outcome,
                                      outcome.lowerBand,outcome.upperBand)
                            )
 
-    ## merge with convariates ####
+    ## merge with covariates ####
     outcome[, row := 1:.N]
-    if(group.by == "covariates"){
+    keep.col <- "row"
+    if(!is.null(newdata)){ ## predicted survival/risk/average treatment effect
         cov.names <- names(newdata)
+
+        ## used individually
         newdata <- newdata[, (cov.names) := lapply(cov.names,function(col){
             if (is.numeric(.SD[[col]]))
-                paste0(col,"=",round(.SD[[col]],digits)) else paste0(col,"=",.SD[[col]])})]
-        outcome[, ("covariates") := interaction(newdata,sep = " ")]
-    }else if(group.by == "strata"){
-        outcome[, strata := strata]
+                round(.SD[[col]],digits) else .SD[[col]]})]
+        outcome <- cbind(outcome,newdata)
+
+        ## keep name of the covariate for when merging all covariates together
+        newdata <- newdata[, (cov.names) := lapply(cov.names,function(col){
+            paste0(col,"=",.SD[[col]])})]
+        outcome[, c("covariates") := interaction(newdata,sep = " ")]
+        
+        keep.col <- c(keep.col,if(!is.null(cov.names)){"covariates"},cov.names)
     }
-    
+
+    if(!is.null(strata)){ ##  baseline survival/risk
+        outcome[, strata := strata]
+        if(!is.null(attr(strata,"covariates"))){
+            outcome[,c(names(attr(strata,"covariates"))) := attr(strata,"covariates") ]
+        }
+        keep.col <- c(keep.col,"strata",names(attr(strata,"covariates")))
+    }
+
     ## reshape to long format ####
-    dataL <- melt(outcome, id.vars = union("row",group.by),
-                  measure = patterns(pattern),
-                  variable.name = "time", value.name = gsub("_","",pattern))
+    dataL <- data.table::melt(outcome, id.vars = keep.col,
+                              measure = patterns(pattern),
+                              variable.name = "time", value.name = gsub("_","",pattern))
 
     dataL[, time := as.numeric(as.character(factor(time, labels = time.names)))]
     dataL <- dataL[!is.na(dataL[[name.outcome]])]
@@ -379,23 +498,13 @@ predict2plot <- function(dataL, name.outcome,
                          ci, band, group.by, smooth,                        
                          conf.level, alpha, xlab, ylab,
                          smoother = NULL, formula.smoother = NULL, first.derivative = FALSE,
-                         size.estimate = 1.5, size.point = 3, size.ci = 1.1, size.band = 1.1, n.sim = 250){
-
+                         size.estimate = 1.5, size.point = 3, size.ci = 1.1, size.band = 1.1, shape.point = c(3,18), n.sim = 250){
+    
     .GRP <- NULL ## [:: for CRAN CHECK::]
-
     if(first.derivative && (smooth==FALSE)){
         stop("Set argument \'smooth\' to TRUE when \'first.derivative\' is TRUE. \n")
     }
-    
     dataL <- data.table::copy(dataL)
-    ## duplicate observations to obtain step curves ####
-    keep.cols <- unique(c("time",name.outcome,"row",group.by))
-    if(ci){
-        keep.cols <- c(keep.cols,"lowerCI","upperCI")
-    }
-    if(band){
-        keep.cols <- c(keep.cols,"lowerBand","upperBand")
-    }
 
     ## set at t- the value of t-1
     vec.outcome <- name.outcome
@@ -405,12 +514,12 @@ predict2plot <- function(dataL, name.outcome,
     if(band){
         vec.outcome <- c(vec.outcome,"lowerBand","upperBand")
     }
-    dataL[,c("timeRight") := c(.SD$time[2:.N]-1e-12,.SD$time[.N]+1e-12), by = "row"] 
-
+    group.by2 <- unique(c(group.by,"row"))
+    dataL[,c("timeRight") := c(.SD$time[2:.N]-1e-12,.SD$time[.N]+1e-12), by = group.by2] 
     dataL[,c(group.by) := as.factor(.SD[[group.by]])]
-
+    
     ## smooth ####
-    if(smooth){
+    if(smooth>=1){
         requireNamespace("mgcv",quietly=FALSE)
         if(is.null(smoother)){
             tol <- 1e-12
@@ -422,7 +531,7 @@ predict2plot <- function(dataL, name.outcome,
             }else{ 
                 smoother <- function(formula, data){
                     out <- try(do.call(scam::scam, args = list(formula = formula, data = data)))
-                    if(inherits(out,"try-error")){
+                    if(inherits(x=out,what="try-error")){
                         out <- do.call(mgcv::gam, args = list(formula = update(formula, ".~s(time)"), data = data))
                     }
                     return(out)
@@ -453,7 +562,7 @@ predict2plot <- function(dataL, name.outcome,
                         data2 <- data.table::copy(data)
                         data2[, c(name.outcome) := mvtnorm::rmvnorm(1, mean = data[[name.outcome]], sigma = Sigma)[1,]]                        
                         iModel <- try(do.call(smoother, args = list(formula = ff,data = data2)))
-                        if(inherits(iModel,"try-error")){
+                        if(inherits(x=iModel,what="try-error")){
                             return(rep(NA, length(data$time)))
                         }else{
                             return(numDeriv::grad(function(x){predict(iModel, newdata = data.frame(time = x), type = "response")}, x = data$time))
@@ -491,19 +600,23 @@ predict2plot <- function(dataL, name.outcome,
 
     gg.base <- ggplot2::ggplot(data = dataL, mapping = ggplot2::aes(group = row))
     if(band){ ## confidence band
-        if(smooth){
+        if(smooth>0){
             if(!is.na(alpha)){
-                gg.base <- gg.base + ggplot2::geom_ribbon(ggplot2::aes_string(x = "time", ymin = "lowerBand.smooth", ymax = "upperBand.smooth"), alpha = alpha)
+                gg.base <- gg.base + ggplot2::geom_ribbon(eval(parse(text = paste0(
+                                                                         "ggplot2::aes(x = time, ymin = lowerBand.smooth, ymax = upperBand.smooth, group = ",group.by,")"))),
+                                                          alpha = alpha)
             }else{
-                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0("ggplot2::aes(x = ","time",", y = lowerBand.smooth, color = ",group.by,", linetype = \"band\")"))),
+                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
+                                                                       "ggplot2::aes(x = time, y = lowerBand.smooth, group = ",group.by,", color = ",group.by,", linetype = \"band\")"))),
                                                         size = size.band)
-                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0("ggplot2::aes(x = ","time",", y = upperBand.smooth, color = ",group.by,", linetype = \"band\")"))),
+                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
+                                                                       "ggplot2::aes(x = time, y = upperBand.smooth, group = ",group.by, ", color = ",group.by,", linetype = \"band\")"))),
                                                         size = size.band)
             }
         }else{
             if(!is.na(alpha)){
                 gg.base <- gg.base + ggplot2::geom_rect(ggplot2::aes_string(xmin = "time", xmax = "timeRight", ymin = "lowerBand", ymax = "upperBand",
-                                                            fill = "labelBand"), linetype = 0, alpha = alpha)
+                                                                            fill = "labelBand"), linetype = 0, alpha = alpha)
                 gg.base <- gg.base + scale_fill_manual("", values="grey12")        
             }else{
                 gg.base <- gg.base + ggplot2::geom_segment(ggplot2::aes_string(x = "time", y = "lowerBand", xend = "timeRight", yend = "lowerBand", color = "\"band\""),
@@ -513,17 +626,18 @@ predict2plot <- function(dataL, name.outcome,
             }
         }
     }
-    
     if(ci){ ## confidence interval
-        if(smooth){
+        if(smooth>0){
             if(!is.na(alpha)){
                 gg.base <- gg.base + ggplot2::geom_errorbar(ggplot2::aes_string(x = "time", ymin = "lowerCI.smooth", ymax = "upperCI.smooth", linetype = "labelCI"),
                                                             width = size.ci)
                 gg.base <- gg.base + ggplot2::scale_linetype_manual("",values=setNames(1,labelCI))
             }else{
-                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0("ggplot2::aes(x = time, y = lowerCI.smooth, color = ",group.by,", linetype = \"ci\")"))),
+                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
+                                                                       "ggplot2::aes(x = time, y = lowerCI.smooth, group = ",group.by,", color = ",group.by,", linetype = \"ci\")"))),
                                                         size = size.ci)
-                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0("ggplot2::aes(x = time, y = upperCI.smooth, color = ",group.by,", linetype = \"ci\")"))),
+                gg.base <- gg.base + ggplot2::geom_line(eval(parse(text = paste0(
+                                                                       "ggplot2::aes(x = time, y = upperCI.smooth, group = ",group.by,", color = ",group.by,", linetype = \"ci\")"))),
                                                         size = size.ci)
 
             }
@@ -542,8 +656,8 @@ predict2plot <- function(dataL, name.outcome,
         }
     }
     ## estimate
-    if(smooth){
-        gg.base <- gg.base + ggplot2::geom_line(mapping = ggplot2::aes_string(x = "time", y = paste0(name.outcome,".smooth"), color = group.by),
+    if(smooth>0){
+        gg.base <- gg.base + ggplot2::geom_line(mapping = ggplot2::aes_string(x = "time", y = paste0(name.outcome,".smooth"), group = group.by, color = group.by),
                                                 size = size.estimate)
     }else{
         gg.base <- gg.base + ggplot2::geom_segment(mapping = ggplot2::aes_string(x = "timeRight", y = name.outcome, xend = "time", yend = name.outcome, color = group.by),
@@ -552,7 +666,7 @@ predict2plot <- function(dataL, name.outcome,
             dataL$status <- as.character(dataL$status)
             gg.base <- gg.base + ggplot2::geom_point(data = na.omit(dataL),
                                                      mapping = ggplot2::aes_string(x = "time", y = name.outcome, color = group.by, shape = "status"), size = size.point)
-            gg.base <- gg.base + ggplot2::scale_shape_manual(breaks = c(0,1), values=c(3,18), labels = c("censoring","event"))
+            gg.base <- gg.base + ggplot2::scale_shape_manual(breaks = c(0,1), values = shape.point, labels = c("censoring","event"))
         }else{
             gg.base <- gg.base + ggplot2::geom_point(data = dataL,
                                                      mapping = ggplot2::aes_string(x = "time", y = name.outcome, color = group.by), size = size.point)
@@ -567,7 +681,7 @@ predict2plot <- function(dataL, name.outcome,
         uniqueObs <- unique(dataL$row)
 
         if(length(uniqueObs)==1){
-            gg.base <- gg.base + ggplot2::scale_color_discrete(guide=FALSE)
+            gg.base <- gg.base + ggplot2::scale_color_discrete(guide="none")
         }
     }
 
@@ -591,8 +705,10 @@ predict2plot <- function(dataL, name.outcome,
                                              group = ggplot2::guide_legend(order = 3)
                                              )
     }
-    gg.base <- gg.base + ggplot2::xlab(xlab) + ggplot2::ylab(ylab) + ggplot2::coord_cartesian(xlim = c(0,max(dataL$timeRight)))
-    
+    gg.base <- gg.base + ggplot2::xlab(xlab) + ggplot2::ylab(ylab)
+    if(name.outcome != "lp"){
+        gg.base <- gg.base + ggplot2::coord_cartesian(xlim = c(0,max(dataL$timeRight)))
+    }
     ## export
     ls.export <- list(plot = gg.base,
                       data = dataL)
