@@ -154,7 +154,22 @@ predictCox <- function(object,
                        diag = FALSE,
                        average.iid = FALSE,
                        store.iid = "full"){
-  
+
+    call <- match.call()
+    ## centering
+    if(!is.null(newdata)){
+        if(inherits(centered,"data.frame")){
+            df.reference <- centered
+            centered2 <- TRUE ## for the linear predictor of the hazard
+        }else{
+            df.reference <- NULL
+            centered2 <- centered ## for the linear predictor of the hazard
+        }
+        centered <- TRUE ## for the linear predictor of the baseline hazard
+    }else{
+        centered2 <- FALSE
+    }
+    
     ## ** Extract elements from object
     if (missing(times)) {
         nTimes <- 0
@@ -185,7 +200,7 @@ predictCox <- function(object,
     is.strata <- infoVar$is.strata
     object.levelStrata <- levels(object.modelFrame$strata) ## levels of the strata variable
     nStrata <- length(object.levelStrata) ## number of strata
-    nVar <- length(infoVar$lpvars) ## number of variables in the linear predictor
+    nVar.lp <- length(infoVar$lpvars) ## number of variables in the linear predictor
 
     ## ** normalize model frame
     ## convert strata to numeric
@@ -212,14 +227,14 @@ predictCox <- function(object,
         attr(times,"etimes.max") <- NULL
         attr(times.sorted,"etimes.max") <- etimes.max
     }else if(is.strata){ ## based on the data
-        if(nVar==0){
+        if(nVar.lp==0){
             iDTtempo <- object.modelFrame[, .SD[which.max(.SD$stop)], by = "strata.num"]
             etimes.max <- iDTtempo[,if(.SD$status==1){1e12}else{.SD$stop}, by = "strata.num"][[2]]
         }else{
             etimes.max <- object.modelFrame[, max(.SD$stop), by = "strata.num"][[2]]
         }
     }else{
-        if(nVar==0 && (utils::tail(object.modelFrame$status,1)==1)){ ## no covariates and ends by a death
+        if(nVar.lp==0 && (utils::tail(object.modelFrame$status,1)==1)){ ## no covariates and ends by a death
             etimes.max <- 1e12
         }else{
             etimes.max <- max(object.modelFrame[["stop"]])
@@ -232,8 +247,14 @@ predictCox <- function(object,
         stop("Missing (NA) values in argument \'times\' are not allowed.\n")
     }
     type <- tolower(type)
-    if(any(type %in% c("hazard","cumhazard","survival") == FALSE)){
-        stop("type can only be \"hazard\", \"cumhazard\" or/and \"survival\" \n") 
+    if(any(type %in% c("lp","hazard","cumhazard","survival") == FALSE)){
+        stop("type can only be \"lp\", \"hazard\", \"cumhazard\" or/and \"survival\" \n") 
+    }
+    if(is.null(newdata) && "lp" %in% type){
+        stop("Cannot evaluate the linear predictor when argument \'lp\' is missing. \n")
+    }
+    if(length(times)>1 && "lp" %in% type){
+        stop("Cannot evaluate the linear predictor when there are multiple timepoints. \n")
     }
     ## predictCox is not compatible with all coxph/cph object (i.e. only handle only simple cox models)
     if(!is.null(object$weights) && !all(object$weights==1)){
@@ -249,18 +270,20 @@ predictCox <- function(object,
     if(object.baseEstimator == "exact"){
         stop("Prediction with exact handling of ties is not implemented.\n")
     }
+    if(!is.null(object$call$tt)){
+        stop("predictCox does not know how to handle time varying effects.\n") 
+    }
     ## convergence issue
     if(!is.null(coef(object)) && any(is.na(coef(object)))){
         print(coef(object))
-        stop("Incorrect object",
-             "One or several model parameters have been estimated to be NA \n")
+        stop("One or several parameters of the regression model have no value, i.e., a value 'NA'.\n")
     }
     ## prediction 
     if (missing(newdata) && (se || iid || average.iid)){
         stop("Argument 'newdata' is missing. Cannot compute standard errors in this case.")
     }
     if(!is.null(newdata)){
-        if(missing(times) || nTimes[1]==0){
+        if(nTimes[1]==0 && !identical(as.character(type),"lp")){
             stop("Time points at which to evaluate the predictions are missing \n")
         }
         if(!is.vector(times)){
@@ -338,7 +361,6 @@ predictCox <- function(object,
                            predtimes = times.sorted,
                            cause = 1,
                            Efron = (object.baseEstimator == "efron"))
-
     ## restaure strata levels
     Lambda0$strata <- factor(Lambda0$strata, levels = 0:(nStrata-1), labels = object.levelStrata)
   
@@ -356,7 +378,7 @@ predictCox <- function(object,
         if (keep.times==FALSE){
             Lambda0$times <- NULL
         }
-        if (keep.strata[[1]]==FALSE ||(is.null(match.call()$keep.strata) && !is.strata)){
+        if (keep.strata[[1]]==FALSE ||(is.null(call$keep.strata) && !is.strata)){
             Lambda0$strata <- NULL
         }
         if( keep.newdata[1]==TRUE){
@@ -368,7 +390,8 @@ predictCox <- function(object,
                          type = type,
                          nTimes = nTimes,
                          baseline = TRUE,
-                         nVar = nVar)
+                         var.lp = infoVar$lpvars.original,
+                         var.strata = infoVar$stratavars.original)
         if(keep.infoVar){
             add.list$infoVar <- infoVar
         }
@@ -384,17 +407,23 @@ predictCox <- function(object,
       ## newdata <- copy(newdata)
       setDT(newdata)
 
-      new.eXb <- exp(coxLP(object, data = newdata, center = FALSE))
-
-      new.strata <- coxStrata(object, data = newdata, 
-                              sterms = infoVar$strata.sterms, 
-                              strata.vars = infoVar$stratavars, 
-                              strata.levels = infoVar$strata.levels)
+        Xb <- coxLP(object, data = newdata, center = FALSE)
+        if ("lp" %in% type){
+            out$lp <- cbind(Xb)
+            lp.iid <- centered2 ## when ask for centered then we need the iid for exporting the correct se
+        }else{
+            lp.iid <- FALSE
+        }
+        new.eXb <- exp(Xb)
+        new.strata <- coxStrata(object, data = newdata, 
+                                sterms = infoVar$strata.sterms, 
+                                strata.vars = infoVar$stratavars, 
+                                strata.levels = infoVar$strata.levels)
 
       new.levelStrata <- levels(droplevels(new.strata))
 
       ## *** subject specific hazard
-      if (is.strata==FALSE){
+      if (is.strata==FALSE && !identical(as.character(type),"lp")){
           if(diag){
               if(needOrder){
                   iTimes <- prodlim::sindex(jump.times = Lambda0$times, eval.times = times.sorted[oorder.times])
@@ -426,7 +455,7 @@ predictCox <- function(object,
               }
           }              
            
-      }else{ 
+      }else if(!identical(as.character(type),"lp")){ 
           ## initialization
           if ("hazard" %in% type){
               out$hazard <- matrix(0, nrow = new.n, ncol = nTimes*(1-diag)+diag)
@@ -482,15 +511,16 @@ predictCox <- function(object,
       }
     
     if(se[[1]] || band[[1]] || iid[[1]] || average.iid[[1]]){
-        if(nVar > 0){
-            ## use prodlim to get the design matrix
-            new.LPdata <- prodlim::model.design(infoVar$lp.sterms,
-                                                data = newdata,
-                                                specialsFactor = TRUE,
-                                                dropIntercept = TRUE)$design
+        if(nVar.lp > 0){
+            ## get the (new) design matrix
+            new.LPdata <- model.matrix(object, data = newdata)
             if(NROW(new.LPdata)!=NROW(newdata)){
-                stop("NROW of the design matrix and newdata differ \n",
-                     "maybe because newdata contains NA values \n")
+                stop("NROW of the design matrix and newdata differ. \n",
+                     "Maybe because newdata contains NA values \n")
+            }
+            if(any(sort(colnames(new.LPdata))!=sort(names(coef(object))))){
+                stop("Names of the design matrix and model parameters differ. \n",
+                     "Possible error in model.matrix due to special operator in the formula. \n")
             }
         }else{
             new.LPdata <- matrix(0, ncol = 1, nrow = new.n)
@@ -505,7 +535,7 @@ predictCox <- function(object,
         }
 
         ## Computation of the influence function and/or the standard error
-        export <- c("iid"[(iid+band)>0],"se"[(se+band)>0],"average.iid"[average.iid==TRUE])
+        export <- c("iid"[(iid+band+lp.iid)>0],"se"[(se+band)>0],"average.iid"[average.iid==TRUE])
         if(!is.null(attr(average.iid,"factor"))){
             if(diag){
                 attr(export,"factor") <- attr(average.iid,"factor")
@@ -539,12 +569,15 @@ predictCox <- function(object,
                            new.LPdata = new.LPdata,
                            new.strata = new.strata,
                            new.survival = if(diag){out$survival}else{out$survival[,order.times,drop=FALSE]},
-                           nVar = nVar, 
+                           nVar.lp = nVar.lp, 
                            export = export,
                            store.iid = store.iid)
 
         ## restaure orginal time ordering
         if((iid+band)>0){
+            if ("lp" %in% type){
+                out$lp.iid <- outSE$lp.iid
+            }
             if ("hazard" %in% type){
                 if (needOrder[1] && (diag[1] == FALSE))
                     out$hazard.iid <- outSE$hazard.iid[,oorder.times,,drop=0L]
@@ -565,6 +598,9 @@ predictCox <- function(object,
             }
         }
         if(average.iid == TRUE){
+            if("lp" %in% type){
+                out$lp.average.iid <- outSE$lp.average.iid
+            }
             if ("hazard" %in% type){
                 if (needOrder && (diag[1] == FALSE)){
                     if(is.list(outSE$hazard.average.iid)){
@@ -613,6 +649,9 @@ predictCox <- function(object,
 
         }
         if((se+band)>0){
+            if("lp" %in% type){
+                out$lp.se <- outSE$lp.se
+            }
             if ("cumhazard" %in% type){
                 if (needOrder && (diag[1] == FALSE)){
                     out$cumhazard.se <- outSE$cumhazard.se[,oorder.times,drop=0L]
@@ -630,33 +669,79 @@ predictCox <- function(object,
         }      
     }
 
-      ## ** add information to the predictions
-      add.list <- list(lastEventTime = etimes.max,
-                       se = se,
-                       band = band,
-                       type = type,
-                       diag = diag,
-                       nTimes = nTimes,
-                       baseline = FALSE,
-                       nVar = nVar)
-      if (keep.times==TRUE){
-          add.list$times <- times
-      }
-      if (is.strata[1] && keep.strata[1]==TRUE){
-          add.list$strata <- new.strata
-      }
+        ## ** substract reference
+        if("lp" %in% type && centered2){
+            if(is.null(df.reference)){
+                data <- try(eval(object$call$data), silent = TRUE)
+                if(inherits(x=data,what="try-error")){
+                    stop("Could not evaluate the dataset used to fit the model to define a reference level. \n",
+                         "Set argument \'centered\' to FALSE or to a data.frame definining the reference level. \n")
+                }
+                var.original <- infoVar$lpvars.original
+                
+                ls.ref <- lapply(var.original, function(iVar){
+                    if(is.numeric(data[[iVar]])){
+                        return(unname(mean(data[[iVar]])))
+                    }else if(is.factor(data[[iVar]])){
+                        return(unname(factor(levels(data[[iVar]])[1],levels(data[[iVar]]))))
+                    }else if(is.character(data[[iVar]])){
+                        return(unname(sort(unique(data[[iVar]])[1])))
+                    }
+                })
+                df.reference <- as.data.frame(setNames(ls.ref,var.original))
+            }
+
+            ls.args <- as.list(call)[-1]
+            ls.args$newdata <- df.reference
+            ls.args$centered <- FALSE
+            ls.args$type <- "lp"
+            ls.args$se <- (se+band>0)
+            ls.args$iid <- (iid+se+band>0)
+            ls.args$band <- FALSE
+            ls.args$confint <- FALSE
+            outRef <- do.call(predictCox, args = ls.args)
+
+            out$lp <- out$lp - as.double(outRef$lp)
+            if(band[1] || se[1]){
+                out$lp.se <- cbind(sqrt(colSums(colCenter_cpp(outSE$lp.iid, outRef$lp.iid)^2))) ## use outSe$lp.iid instead of out$lp.iid for when not exporting the iid
+            }
+            if(band[1] || iid[1]){
+                out$lp.iid <- colCenter_cpp(out$lp.iid, outRef$lp.iid)
+            }
+            
+        }
+        
+        ## ** add information to the predictions
+        add.list <- list(lastEventTime = etimes.max,
+                         se = se,
+                         band = band,
+                         type = type,
+                         diag = diag,
+                         nTimes = nTimes,
+                         baseline = FALSE,
+                         var.lp = infoVar$lpvars.original,
+                         var.strata = infoVar$stratavars.original)
+        if (keep.times==TRUE){
+            add.list$times <- times
+        }
+        if (is.strata[1] && keep.strata[1]==TRUE){
+            add.list$strata <- new.strata
+        }
       
       if( keep.infoVar){
           add.list$infoVar <- infoVar
       }
       all.covars <- c(infoVar$stratavars.original,infoVar$lpvars.original)
-      if( keep.newdata[1]==TRUE && length(all.covars)>0){
-          add.list$newdata <- newdata[, all.covars, with = FALSE]
+        if( keep.newdata[1]==TRUE && length(all.covars)>0){
+            if (data.table::is.data.table(newdata))
+                add.list$newdata <- newdata[, all.covars, with = FALSE]
+            else
+                add.list$newdata <- newdata[, all.covars, drop=FALSE]
       }
       out[names(add.list)] <- add.list
       class(out) <- "predictCox"
 
-      ## ** confidence intervals/bands
+        ## ** confidence intervals/bands
       if(confint){
           out <- stats::confint(out)
       }
