@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: jun 21 2018 (15:10) 
 ## Version: 
-## Last-Updated: sep 25 2019 (09:37) 
-##           By: Brice Ozenne
-##     Update #: 19
+## Last-Updated: Jan 12 2022 (11:29) 
+##           By: Thomas Alexander Gerds
+##     Update #: 25
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -54,10 +54,10 @@ set.seed(10)
 dt <- sampleData(50, outcome = "survival")[,.(time,event,X1,X2,X6)]
 
 test_that("[predictCoxPL] diag no strata", {
-    e.coxph <- coxph(Surv(time, event) ~ X1*X6, data = dt, y = TRUE, x = TRUE)
+    e.coxph <- coxph(Surv(time, event) ~ X1*X6, data = dt, y = TRUE, x = TRUE,)
 
     GS0 <- predictCox(e.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE)
-    GS <- predictCoxPL(e.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE)
+    suppressWarnings(GS <- predictCoxPL(e.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE))
     test <- predictCoxPL(e.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE, diag = TRUE)
 
     ## check hazard/survival
@@ -67,19 +67,19 @@ test_that("[predictCoxPL] diag no strata", {
 
     ## check iid for survival
     expect_equal(GS0$survival.iid, GS$survival.iid) ## same predictCox / predictCoxPL
-    expect_equal(GS$survival.iid[1,1,], test$survival.iid[1,1,]) ## same when using diag
-    expect_equal(apply(GS$survival.iid,3,diag), test$survival.iid[,1,])
-
+    expect_equal(GS$survival.iid[,1,1], test$survival.iid[,1,1]) ## same when using diag
+    expect_equal(t(apply(GS$survival.iid,1,diag)), test$survival.iid[,1,])
+    
     ## check average.iid for survival
     expect_equal(GS0$survival.average.iid, GS$survival.average.iid) ## same predictCox / predictCoxPL
-    expect_equal(colMeans(apply(GS$survival.iid,3,diag)), test$survival.average.iid[,1])
+    expect_equal(rowMeans(t(apply(GS$survival.iid,1,diag))), test$survival.average.iid[,1])
 })
 
 test_that("[predictCoxPL] diag strata", {
     eS.coxph <- coxph(Surv(time, event) ~ strata(X1) + X6, data = dt, y = TRUE, x = TRUE)
 
     GS0 <- predictCox(eS.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE)
-    GS <- predictCoxPL(eS.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE)
+    suppressWarnings(GS <- predictCoxPL(eS.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE))
     test <- predictCoxPL(eS.coxph, newdata = dt, times = dt$time, iid = TRUE, average.iid = TRUE, se = FALSE, diag = TRUE)
 
     ## check hazard/survival
@@ -89,12 +89,12 @@ test_that("[predictCoxPL] diag strata", {
 
     ## check iid for survival
     expect_equal(GS0$survival.iid, GS$survival.iid) ## same predictCox / predictCoxPL
-    expect_equal(GS$survival.iid[1,1,], test$survival.iid[1,1,]) ## same when using diag
-    expect_equal(apply(GS$survival.iid,3,diag), test$survival.iid[,1,])
+    expect_equal(GS$survival.iid[,1,1], test$survival.iid[,1,1]) ## same when using diag
+    expect_equal(t(apply(GS$survival.iid,1,diag)), test$survival.iid[,1,])
 
     ## check average.iid for survival
     expect_equal(GS0$survival.average.iid, GS$survival.average.iid) ## same predictCox / predictCoxPL
-    expect_equal(colMeans(apply(GS$survival.iid,3,diag)), test$survival.average.iid[,1])
+    expect_equal(rowMeans(t(apply(GS$survival.iid,1,diag))), test$survival.average.iid[,1])
 })
 
 ######################################################################
